@@ -352,9 +352,9 @@ _tp_account_manager_update_most_available_presence (TpAccountManager *manager)
 
   if (account == NULL)
     {
-      priv->most_available_presence = TP_CONNECTION_PRESENCE_TYPE_OFFLINE;
-      priv->most_available_status = g_strdup ("offline");
-      priv->most_available_status_message = g_strdup ("");
+      priv->most_available_presence = presence;
+      priv->most_available_status = NULL;
+      priv->most_available_status_message = NULL;
       return;
     }
 
@@ -1280,7 +1280,7 @@ tp_account_manager_create_account_async (TpAccountManager *manager,
  * object, with the %TP_ACCOUNT_FEATURE_CORE feature ready on it.
  *
  * The caller must keep a ref to the returned object using g_object_ref() if
- * it is to be kept beyond the lifetime of @result.
+ * it is to be kept.
  *
  * Returns: (transfer none): a new #TpAccount which was just created on
  *  success, otherwise %NULL
