@@ -304,12 +304,6 @@ tp_simple_password_manager_channel_closed_cb (GObject *chan,
   tp_clear_object (&manager->priv->channel);
 }
 
-static void
-free_gstring (gpointer p)
-{
-  g_string_free (p, TRUE);
-}
-
 static void
 tp_simple_password_manager_channel_finished_cb (
     TpBasePasswordChannel *channel,
@@ -331,7 +325,7 @@ tp_simple_password_manager_channel_finished_cb (
   else
     {
       g_simple_async_result_set_op_res_gpointer (
-          result, g_boxed_copy (G_TYPE_GSTRING, str), free_gstring);
+          result, (gpointer) str, NULL);
     }
 
   g_simple_async_result_complete (result);
