@@ -13,6 +13,7 @@
 #include <telepathy-glib/defs.h>
 
 #include "tests/lib/simple-account-manager.h"
+#include "tests/lib/util.h"
 
 typedef struct {
     GFunc action;
@@ -22,9 +23,8 @@ typedef struct {
 typedef struct {
     GMainLoop *mainloop;
     TpDBusDaemon *dbus;
-    DBusGConnection *bus;
 
-    SimpleAccountManager *service /* initialized in prepare_service */;
+    TpTestsSimpleAccountManager *service /* initialized in prepare_service */;
     TpAccountManager *am;
     TpAccount *account;
     gboolean prepared /* The result of prepare_finish */;
@@ -127,8 +127,7 @@ setup (Test *test,
 
   test->mainloop = g_main_loop_new (NULL, FALSE);
 
-  test->dbus = tp_dbus_daemon_dup (NULL);
-  g_assert (test->dbus != NULL);
+  test->dbus = tp_tests_dbus_daemon_dup_or_die ();
 
   test->am = NULL;
   test->timeout_id = 0;
@@ -139,17 +138,15 @@ static void
 setup_service (Test *test,
     gconstpointer data)
 {
-
   setup (test, data);
 
-  test->bus = tp_get_bus ();
-
   g_assert (tp_dbus_daemon_request_name (test->dbus,
           TP_ACCOUNT_MANAGER_BUS_NAME, FALSE, &test->error));
 
-  test->service = g_object_new (SIMPLE_TYPE_ACCOUNT_MANAGER, NULL);
-  dbus_g_connection_register_g_object (test->bus, TP_ACCOUNT_MANAGER_OBJECT_PATH,
-      (GObject *) test->service);
+  test->service = tp_tests_object_new_static_class (
+      TP_TESTS_TYPE_SIMPLE_ACCOUNT_MANAGER, NULL);
+  tp_dbus_daemon_register_object (test->dbus, TP_ACCOUNT_MANAGER_OBJECT_PATH,
+      test->service);
 }
 
 static void
@@ -168,6 +165,10 @@ teardown (Test *test,
     }
   g_queue_free (test->script);
   test->script = NULL;
+
+  /* make sure any pending things have happened */
+  tp_tests_proxy_run_until_dbus_queue_processed (test->dbus);
+
   g_object_unref (test->dbus);
   test->dbus = NULL;
   g_main_loop_unref (test->mainloop);
@@ -182,7 +183,7 @@ teardown_service (Test *test,
   g_assert (
       tp_dbus_daemon_release_name (test->dbus, TP_ACCOUNT_MANAGER_BUS_NAME,
                                &test->error));
-  dbus_g_connection_unregister_g_object (test->bus, G_OBJECT (test->service));
+  tp_dbus_daemon_unregister_object (test->dbus, test->service);
   g_object_unref (test->service);
   test->service = NULL;
   teardown (test, data);
@@ -425,8 +426,8 @@ register_service_action (gpointer script_data,
 {
   Test *test = (Test *) script_data;
 
-  dbus_g_connection_register_g_object (test->bus, TP_ACCOUNT_MANAGER_OBJECT_PATH,
-      (GObject *) test->service);
+  tp_dbus_daemon_register_object (test->dbus, TP_ACCOUNT_MANAGER_OBJECT_PATH,
+      test->service);
 
   script_continue (test);
 }
@@ -487,7 +488,7 @@ static void
 test_prepare_destroyed (Test *test,
     gconstpointer data G_GNUC_UNUSED)
 {
-  dbus_g_connection_unregister_g_object (test->bus, G_OBJECT (test->service));
+  tp_dbus_daemon_unregister_object (test->dbus, test->service);
   test_prepare (test, data);
   script_append_action (test, assert_failed_action, NULL);
   script_append_action (test, register_service_action, NULL);
@@ -525,6 +526,7 @@ int
 main (int argc,
     char **argv)
 {
+  tp_tests_abort_after (10);
   g_test_init (&argc, &argv, NULL);
   g_test_bug_base ("http://bugs.freedesktop.org/show_bug.cgi?id=");
 
